from abc import abstractmethod

class _BaseChat:
    @abstractmethod
    def chat(self, prompt: str, *args, **kwargs) -> str:
        """
        Prompts an LLM.
        :param text: The text to prompt the LLM with.
        :return: The generated response from the llm.
        """
        raise NotImplementedError("Please implement this method")


# Factory method for generalized model_hosting_info calling
def chat(prompt: str, model="meta-llama-3-13b", service="socaity", *args, **kwargs) -> str:
    from .llama3 import MetaLLama3_70b, MetaLLama3_8b, MetaLLama3_70b_code_python, MetaLLama3_13b_code, MetaLLama3_70b_instruct
    from .deepseek import DeepSeekR1
    names_cl = {
        "meta-llama-3-8b": MetaLLama3_8b,
        "meta-llama-3-70b": MetaLLama3_70b,
        "meta-llama-3-70b-code": MetaLLama3_70b_code_python,
        "meta-llama-3-13b-code": MetaLLama3_13b_code,
        "meta-llama-3-70b-instruct": MetaLLama3_70b_instruct,
        "deepseek-r1": DeepSeekR1
    }
    names_cl = {k.lower().replace("-", ""): v for k, v in names_cl.items()}
    mdlcl = names_cl.get(model.lower().replace("-", ""), MetaLLama3_8b)

    return mdlcl(service=service).chat(prompt=prompt, *args, **kwargs)



from typing import Union

from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk.fast_sdk import fastSDK, fastJob
from media_toolkit import  AudioFile
from socaity.api.text.audio2text.audio2text import _BaseAudio2Text
from socaity.api.text.audio2text.whisper.whisper_service_client import srvc_whisper, WhisperTasks, WhisperTimeStamp


@fastSDK(api_client=srvc_whisper)
class InsanelyFastWhisper(_BaseAudio2Text):
    """
    Base implementation for SAM 2, the Segment Anything v2 model from Meta.
    """
    @fastJob
    def _whisper(self, job,
        audio: Union[str, bytes, AudioFile],
        task: str = WhisperTasks.TRANSCRIBE,
        language: str = None,
        batch_size: int = 64,
        timestamp: WhisperTimeStamp = WhisperTimeStamp.WORD,
        **kwargs
    ) -> str:
        """
        Create an image description (caption) for the input image.
        """
        tsk = task.value if isinstance(task, WhisperTasks) else task
        ts = timestamp.value if isinstance(timestamp, WhisperTimeStamp) else timestamp
        language = "None" if not language else language

        response = job.request(
            endpoint_route="/transcribe",
            audio=audio,
            task=tsk,
            language=language,
            batch_size=batch_size,
            timestamp=ts,
            **kwargs,
        )
        res = response.get_result()
        if response.error:
            raise Exception(f"Error in whisper: {response.error}")

        return res

    def transcribe(
            self,
            audio: Union[str, bytes, AudioFile],
            language: str = None,
            batch_size: int = 64,
            timestamp: WhisperTimeStamp = WhisperTimeStamp.WORD,
            *args, **kwargs
    ) -> InternalJob:
        """
        :param audio: the audio file to transcribe
        :param language: the language of the audio file. If not given it will be detected.
        :param batch_size: the batch size for the transcription. Bigger batch sizes are faster but require more memory.
        :param timestamp: the timestamp for the transcription. It can be word, sentence or paragraph.
        """
        return self._whisper(
            audio=audio,
            task=WhisperTasks.TRANSCRIBE,
            language=language, batch_size=batch_size, timestamp=timestamp, *args, **kwargs
        )

    def translate(
            self,
            audio: Union[str, bytes, AudioFile],
            batch_size: int = 64,
            timestamp: WhisperTimeStamp = WhisperTimeStamp.WORD,
            *args, **kwargs
    ) -> InternalJob:
        """ will transcribe and translate the audio to english.
        :param audio: the audio file to transcribe and translate
        :param batch_size: the batch size for the transcription. Bigger batch sizes are faster but require more memory.
        :param timestamp: the timestamp for the transcription. It can be word, sentence or paragraph.
        """
        return self._whisper(
            audio=audio,
            task=WhisperTasks.TRANSLATE,
            batch_size=batch_size,
            timestamp=timestamp, *args, **kwargs
        )



from abc import abstractmethod
from typing import Union, List

from fastsdk.jobs.threaded.internal_job import InternalJob
from media_toolkit import ImageFile


class _BaseSegmentation:
    """
    Base implementation for segmentation models, the Segment Anything v2 model from Meta.
    """
    @abstractmethod
    def segment(self,  job, image: Union[str, bytes, ImageFile], *args, **kwargs) -> InternalJob:
        """
        Interface for performing image segmentation.
        :param image: URL or path to the input image.
        """
        raise NotImplementedError("Implement in subclass")


def image_segmentation(
        image: Union[ImageFile, List[ImageFile], str, List[str]],
        model="sam2", service="socaity", *args, **kwargs) -> Union[ImageFile, List[ImageFile], dict, None]:

    from .sam2 import Sam2
    names_cl = {
        "sam2": Sam2,
    }
    names_cl = {k.lower().replace("-", ""): v for k, v in names_cl.items()}
    mdlcl = names_cl.get(model.lower().replace("-", ""), Sam2)
    return mdlcl(service=service).segment(image=image, *args, **kwargs)



import time
from typing import Union
import numpy as np
from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk import fastSDK, fastJob, MediaFile
from .speech_craft_service_client import srvc_speechcraft
from socaity.api.audio.text2voice.i_text2voice import _BaseText2Voice
from socaity.api.audio.voice2voice.i_voice2voice import _BaseVoice2Voice


@fastSDK(api_client=srvc_speechcraft)
class SpeechCraft(_BaseText2Voice, _BaseVoice2Voice):
    """
    SpeechCraft offers Text2Speech, Voice-Cloning and Voice2Voice conversion with the generative audio model_description bark
    SDK for the SpeechCraft https://github.com/SocAIty/SpeechCraft fast-task-api service.
    """
    def text2voice(
            self,
            text: str,
            voice: Union[str, bytes, MediaFile] = "en_speaker_3",
            semantic_temp: float = 0.7,
            semantic_top_k: int = 50,
            semantic_top_p: float = 0.95,
            coarse_temp: float = 0.7,
            coarse_top_k: int = 50,
            coarse_top_p: float = 0.95,
            fine_temp: float = 0.5
    ) -> InternalJob:
        """
        :param text: the text to be converted to speech
        :param voice: the name of the voice to be used. Or an embedding file. Uses the pretrained voices of SpeechCraft
        :param semantic_temp: the temperature for the semantic model_description
        """
        return self._text2voice(
            text=text, voice=voice, semantic_temp=semantic_temp, semantic_top_k=semantic_top_k,
            semantic_top_p=semantic_top_p, coarse_temp=coarse_temp, coarse_top_k=coarse_top_k,
            coarse_top_p=coarse_top_p, fine_temp=fine_temp
        )

    def voice2voice(self, voice_name: str, audio_file: Union[str, bytes], temp: float = 0.7) -> InternalJob:
        return self._voice2voice(voice_name=voice_name, audio_file=audio_file, temp=temp)

    def voice2embedding(self, voice_name: str, audio_file: Union[str, bytes], save: bool = False) -> InternalJob:
        return self._voice2embedding(voice_name=voice_name, audio_file=audio_file, save=save)

    @fastJob
    def _text2voice(
            self,
            job: InternalJob,
            text: str,
            voice: str = "en_speaker_3",
            semantic_temp: float = 0.7,
            semantic_top_k: int = 50,
            semantic_top_p: float = 0.95,
            coarse_temp: float = 0.7,
            coarse_top_k: int = 50,
            coarse_top_p: float = 0.95,
            fine_temp: float = 0.5
         ) -> np.array:
        """
        Swaps a face from source_img to target_img;
        in the manner that the face from source_img is placed on the face from target_img.
        :param source_img: The image containing the face to be swapped. Read with open() -> f.read()
        :param target_img: The image containing the face to be swapped to. Read with open() -> f.read()
        """

        endpoint_route = "text2voice"
        if not isinstance(voice, str):
            endpoint_route = "text2voice_with_embedding"

        endpoint_request = job.request(
            endpoint_route=endpoint_route,
            text=text,
            voice=voice,
            semantic_temp=semantic_temp,
            semantic_top_k=semantic_top_k,
            semantic_top_p=semantic_top_p,
            coarse_temp=coarse_temp,
            coarse_top_k=coarse_top_k,
            coarse_top_p=coarse_top_p,
            fine_temp=fine_temp
        )
        endpoint_request.wait_until_finished()

        if endpoint_request.error is not None:
            raise Exception(f"Error in text2voice: {endpoint_request.error}")

        return endpoint_request.get_result()

    @fastJob
    def _voice2voice(self, job: InternalJob, voice_name: str, audio_file: Union[str, bytes], temp: float = 0.7):
        endpoint_request = job.request("voice2voice", voice_name=voice_name, audio_file=audio_file, temp=temp)
        endpoint_request.wait_until_finished()

        if endpoint_request.error is not None:
            raise Exception(f"Error in voice2voice: {endpoint_request.error}")

        return endpoint_request.get_result()

    @fastJob
    def _voice2embedding(self, job: InternalJob, voice_name: str, audio_file: Union[str, bytes], save: bool = False):
        endpoint_request = job.request("voice2embedding", voice_name=voice_name, audio_file=audio_file, save=save)
        endpoint_request.wait_until_finished()

        if endpoint_request.error is not None:
            raise Exception(f"Error in voice2embedding: {endpoint_request.error}")

        return endpoint_request.get_result()




import time
from typing import Union
import numpy as np
from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk.fast_sdk import fastSDK, fastJob
from media_toolkit import ImageFile, VideoFile
from socaity.api.image.img2img.face2face.face2face_service_client import srvc_face2face


@fastSDK(api_client=srvc_face2face)
class Face2Face:
    def swap_img_to_img(
            self,
            source_img: Union[str, bytes, ImageFile],
            target_img: Union[str, bytes, ImageFile],
            enhance_face_model: Union[str, None] = 'gpen_bfr_512'
    ) -> InternalJob:
        """
        Swaps a face from source_img to target_img;
        in the manner that the face from source_img is placed on the face from target_img.
        :param source_img: Path to the image containing the face to be swapped.
            Or the image itself as bytes (with open(f): f.read()) .
        :param target_img: Path to the image containing the face to be swapped to.
            Or the image itself as bytes (with open(f): f.read()) .
        :param enhance_face_model: The face enhancement model_description to use. Use None for no enhancement.
        """
        return self._swap_img_to_img(
            source_img=source_img, target_img=target_img, enhance_face_model=enhance_face_model
        )

    def swap(
            self,
            faces: Union[str, dict, list],
            media: Union[str, bytes],
            enhance_face_model: Union[str, None] = 'gpen_bfr_512'
    ) -> InternalJob:
        return self._swap(faces=faces, media=media, enhance_face_model=enhance_face_model)

    def add_face(self, face_name: str, image: Union[str, bytes, ImageFile], save: bool = True) -> InternalJob:
        return self._add_face(face_name=face_name, image=image, save=save)

    def swap_video(
            self,
            face_name: str, target_video: Union[str, bytes, VideoFile], include_audio: bool = True,
            enhance_face_model: Union[str, None] = None #'gpen_bfr_512'
        ) -> InternalJob:
        return self._swap_video(face_name=face_name, target_video=target_video, include_audio=include_audio,
                                enhance_face_model=enhance_face_model)

    @fastJob
    def _swap_img_to_img(
            self,
            job: InternalJob,
            source_img: Union[np.array, bytes, str],
            target_img: Union[np.array, bytes, str],
            enhance_face_model: Union[str, None] = 'gpen_bfr_512'
    ) -> Union[ImageFile, None]:
        """
        Swaps a face from source_img to target_img;
        in the manner that the face from source_img is placed on the face from target_img.
        :param source_img: The image containing the face to be swapped. Read with open() -> f.read()
        :param target_img: The image containing the face to be swapped to. Read with open() -> f.read()
        """
        endpoint_request = job.request(
            endpoint_route="swap_img_to_img",
            source_img=source_img, target_img=target_img, enhance_face_model=enhance_face_model
        )
        result = endpoint_request.get_result()
        if endpoint_request.error is not None:
            raise Exception(f"Error in swap_one: {endpoint_request.error}")

        return result

    @fastJob
    def _swap(self, job, faces: Union[str, dict, list], media: bytes, enhance_face_model: str = 'gpen_bfr_512'):
        endpoint_request = job.request("swap", faces=faces, media=media, enhance_face_model=enhance_face_model)
        result = endpoint_request.get_result()
        if endpoint_request.error is not None:
            raise Exception(f"Error in swap: {endpoint_request.error}")

        return result

    @fastJob
    def _add_face(self, job, face_name: str, image: bytes, save: bool = True):
        endpoint_request = job.request("add_face", face_name=face_name, image=image, save=save)
        result = endpoint_request.get_result()
        if endpoint_request.error is not None:
            raise Exception(f"Error in add_reference_face: {endpoint_request.error}")

        return result

    @fastJob
    def _swap_video(
            self, job, face_name: str, target_video: str, include_audio: bool = True,
            enhance_face_model: str = 'gpen_bfr_512'
    ):
        request_result = job.request(
            "swap_video", face_name=face_name, target_video=target_video, include_audio=include_audio,
            enhance_face_model=enhance_face_model
        )

        # update progress bar
        while not request_result.is_finished():
            progress, message = request_result.progress
            job.set_progress(progress, message)
            time.sleep(0)

        if request_result.error is not None:
            raise Exception(f"Error in swap_video: {request_result.error}")

        return request_result.get_result()


if __name__ == "__main__":
    f2f = Face2Face(service="runpod")
    img_1 = "A:\\projects\\_face2face\\face2face\\test\\test_media\\test_face_1.jpg"
    img2 = "A:\\projects\\_face2face\\face2face\\test\\test_media\\test_face_2.jpg"

    job = f2f.swap_img_to_img(img_1, target_img=img2)
    job.run()
    result = job.get_result()
    print(result)



import random
from enum import Enum
from typing import Union, List

from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk.fast_sdk import fastSDK, fastJob
from fastsdk.utils import get_function_parameters_as_dict
from media_toolkit import ImageFile
from socaity.api.text.img2text.blib.blib_service_client import srvc_blip


class BlipTasks(Enum):
    CAPTION = "image_captioning"
    VQA = "visual_question_answering"
    IMAGE_TEXT_MATCHING = "image_text_matching"


@fastSDK(api_client=srvc_blip)
class Blip:
    """
    Base implementation for SAM 2, the Segment Anything v2 model from Meta.
    """
    def __init__(self):
        self._socaity_avatar_reference_image_1 = "https://socaityfiles.blob.core.windows.net/backend-model-meta/socaity_icon_no_shoulders_eyes.ico"

    @fastJob
    def _caption(self, job,
        image: Union[str, bytes, ImageFile],
        **kwargs
    ) -> str:
        """
        Create an image description (caption) for the input image.
        """
        response = job.request(
            endpoint_route="/capture",
            image=image,
            task=BlipTasks.CAPTION.value,
            caption="",
            **kwargs,
        )
        res = response.get_result()
        if response.error:
            raise Exception(f"Error in blip: {response.error}")

        return res

    @fastJob
    def _text_matching(self, job,
        image: Union[str, bytes, ImageFile],
        caption: str,
        **kwargs
    ) -> str:
        """
        Measure how well an image description is matched by the input image.
        """
        response = job.request(
            endpoint_route="/capture",
            image=image,
            task=BlipTasks.IMAGE_TEXT_MATCHING.value,
            caption=caption,
            **kwargs,
        )
        res = response.get_result()
        if response.error:
            raise Exception(f"Error in blip: {response.error}")

        return res

    @fastJob
    def _visual_question_answering(self, job,
        image: Union[str, bytes, ImageFile],
        question: str,
        **kwargs
    ):
        response = job.request(
            endpoint_route="/capture",
            image=image,
            task=BlipTasks.VQA.value,
            question=question,
            **kwargs,
        )
        res = response.get_result()
        if response.error:
            raise Exception(f"Error in blip: {response.error}")

        return res

    def caption(self, image: Union[str, bytes, ImageFile]) -> InternalJob:
        return self._caption(image=image)

    def text_matching(self, image: Union[str, bytes, ImageFile], caption: str) -> InternalJob:
        return self._text_matching(image=image, caption=caption)

    def visual_question_answering(self, image: Union[str, bytes, ImageFile], question: str) -> InternalJob:
        return self._visual_question_answering(image=image, question=question)



from socaity import Face2Face
f2f = Face2Face()

test_face_1 = "test_files/face2face/test_face_1.jpg"
test_face_2 = "test_files/face2face/test_face_2.jpg"
test_face_3 = "test_files/face2face/test_face_3.jpg"
test_video = "test_files/face2face/test_video_ultra_short_short.mp4"

def test_single_face_swap():
    job_swapped = f2f.swap_img_to_img(test_face_1, test_face_2, enhance_face_model=None)
    swapped = job_swapped.get_result()
    swapped.save("test_files/output/face2face/test_face_31_swapped.jpg")
    return swapped


## test embedding face swap
def test_embedding_face_swap():
    ref_face_v_job = f2f.add_face("hagrid", test_face_1, save=True)
    ref_face_vector = ref_face_v_job.get_result()
    job_swapped = f2f.swap(media=test_face_2, faces="hagrid", enhance_face_model="")
    swapped = job_swapped.get_result()
    return swapped
#swap_job = f2f.swap_from_reference_face("hagrid", test_face_2)
#swapped = swap_job.get_result()
# test video swap


def test_video_swap():
    #ref_face_v_job = f2f.add_face(face_name="black_woman", image=test_face_3, save=True)
    #ref_face_vector = ref_face_v_job.get_result()
    swapped_video_job = f2f.swap_video(face_name="black_woman", target_video=test_video, include_audio=True, enhance_face_model="")
    swapped_video = swapped_video_job.get_result()
    return swapped_video

test_single_face_swap()
# test_video_swap()
# test_embedding_face_swap()


from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.api_client import APIClient
from socaity.settings import DEFAULT_SOCAITY_URL, DEFAULT_REPLICATE_URL
from socaity.api.text.chat.llama3.meta_llama3_schema import MetaLlama3_InstructInput

srvc_meta_llama_3_8b_instruct = APIClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/meta-llama-3-8b-instruct",
        "replicate": f"{DEFAULT_REPLICATE_URL}/meta/meta-llama-3-8b-instruct",
        "socaity_local": f"http://localhost:8000/v0/meta-llama-3-8b-instruct",
    },
    service_name="meta-llama-3-8b-instruct",
    model_description=AIModelDescription(
        model_name="meta-llama-3-8b-instruct",
        model_domain_tags=[ModelDomainTag.TEXT]
    ),
)

srvc_meta_llama_3_8b_instruct.add_endpoint(
    endpoint_route="/chat",
    query_params=MetaLlama3_InstructInput(),
    refresh_interval_s=5,
)



srvc_meta_llama_3_70b_instruct = APIClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/meta-llama-3-70b-instruct",
        "replicate": f"{DEFAULT_REPLICATE_URL}/meta/meta-llama-3-70b-instruct",
        "socaity_local": f"http://localhost:8000/v0/meta-llama-3-70b-instruct",
    },
    service_name="meta-llama-3-70b-instruct",
    model_description=AIModelDescription(
        model_name="meta-llama-3-70b-instruct",
        model_domain_tags=[ModelDomainTag.TEXT]
    ),
)

srvc_meta_llama_3_70b_instruct.add_endpoint(
    endpoint_route="/chat",
    query_params=MetaLlama3_InstructInput(),
    refresh_interval_s=5,
)



from socaity import MetaLLama3_70b, MetaLLama3_8b, MetaLLama3_8b_instruct, MetaLLama3_70b_instruct, gather_results

cl_test = [
    MetaLLama3_8b,
    MetaLLama3_70b,
    MetaLLama3_8b_instruct,
    MetaLLama3_70b_instruct
]

providers = ["socaity"]
prompt = "Write a poem with 3 sentences why an SDK is so much better than plain web requests."

jobs = {}
def test_llama_models():
    for provider in providers:
        for mdl in cl_test:
            llama3 = mdl(service=provider)
            fj = llama3.chat(prompt=prompt)
            jobs[llama3.service_client.service_name] = fj

        results = gather_results(list(jobs.values()))
        for mdl, job in zip(jobs.keys(), results):
            print(f"\n{mdl} provider {provider} , result: {job.get_result()}")



from socaity import MetaLLama3_70b_code_python, MetaLLama3_13b_code


cl_code_test = [
    MetaLLama3_13b_code,
    MetaLLama3_70b_code_python
]
code_prompt = "Write a python script to calculate the factorial of a number."

def test_code_models():
    for provider in providers:
        for mdl in cl_test:
            llama3 = mdl(service=provider)
            fj = llama3.chat(prompt=code_prompt)
            jobs[llama3.service_client.service_name] = fj

        results = gather_results(list(jobs.values()))
        for mdl, job in zip(jobs.keys(), results):
            print(f"\n{mdl} provider {provider} , result: {job.get_result()}")


if __name__ == "__main__":
    test_llama_models()
    test_code_models()



from abc import abstractmethod
from typing import Union, List

import media_toolkit as mt

class _BaseVoice2Voice:

    @abstractmethod
    def voice2voice(
            self,
            voice_name: str,
            audio_file: Union[str, bytes],
            *args, **kwargs
    ) -> Union[mt.AudioFile, List[mt.AudioFile], None]:
        """
        Converts text to an image
        :param text: The text to convert to an image
        :return: The image
        """
        raise NotImplementedError("Please implement this method")


# Factory method for generalized model_hosting_info calling
def voice2voice(
        voice_name: str,
        audio_file: Union[str, bytes],
        model="speechcraft",
        service="socaity",
        *args, **kwargs
) -> Union[mt.AudioFile, List[mt.AudioFile], None]:
    if model == "speechcraft":
        from socaity.api.audio.text2voice.speechcraft.speechcraft_api import SpeechCraft
        s = SpeechCraft(service=service)
        return s.voice2voice(voice_name=voice_name, audio_file=audio_file, *args, **kwargs)

    return None



from fastsdk import AudioFile, MediaFile
from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.definitions.service_adress import RunpodServiceAddress
from fastsdk.web.api_client import APIClient
from socaity.settings import DEFAULT_SOCAITY_URL

srvc_speechcraft = APIClient(
    service_name="speechcraft",
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/speechcraft",
        "runpod": "https://api.runpod.ai/v2/esgd0bzgrxtwn1/run",
        "socaity_local": "http://localhost:8000/v0/speechcraft",
        "localhost": "localhost:8009/api",
        "localhost_runpod": RunpodServiceAddress("localhost:8009")
    },
    model_description=AIModelDescription(
        model_name="bark",
        model_domain_tags=[ModelDomainTag.IMAGE, ModelDomainTag.AUDIO]
    )
)
srvc_speechcraft.add_endpoint(
    endpoint_route="/text2voice",
    query_params={
        "text": str,
        "voice": str,
        "semantic_temp": float,
        "semantic_top_k": int,
        "semantic_top_p": float,
        "coarse_temp": float,
        "coarse_top_k": int,
        "coarse_top_p": float,
        "fine_temp": float
     },
    refresh_interval_s=2
)
srvc_speechcraft.add_endpoint(
    endpoint_route="/text2voice_with_embedding",
    query_params={
        "text": str,
        "semantic_temp": float,
        "semantic_top_k": int,
        "semantic_top_p": float,
        "coarse_temp": float,
        "coarse_top_k": int,
        "coarse_top_p": float,
        "fine_temp": float
     },
    file_params={"voice": MediaFile},
    refresh_interval_s=2
)

srvc_speechcraft.add_endpoint(
    endpoint_route="voice2embedding",
    query_params={"voice_name": str, "save": bool},
    file_params={"audio_file": AudioFile},
    refresh_interval_s=2
)
srvc_speechcraft.add_endpoint(
    endpoint_route="voice2voice",
    query_params={
        "voice_name": str,
        "temp": float
    },
    file_params={"audio_file": AudioFile},
    refresh_interval_s=2
)



import os

from socaity import Blip

test_img = "https://socaityfiles.blob.core.windows.net/backend-model-meta/llama3.png"
genai = Blip()
#genai = Blip(service="replicate", api_key=os.getenv("REPLICATE_API_KEY", None))
# fluxs = FluxSchnell(service="socaity_local", api_key=os.getenv("SOCAITY_API_KEY", None))

def test_image_captioning():
    fj = genai.caption(image=test_img)
    generated_text = fj.get_result()
    print(generated_text)

def test_image_text_matching():
    fj = genai.text_matching(image=test_img, caption="a beautiful woman in the sunset")
    matching_score = fj.get_result()
    print(matching_score)

def test_visual_question_answering():
    fj = genai.visual_question_answering(image=test_img, question="what animal is on the image?")
    answer = fj.get_result()
    print(answer)


if __name__ == "__main__":
    test_image_captioning()
    test_image_text_matching()
    test_visual_question_answering()



from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.api_client import APIClient
from socaity.settings import DEFAULT_SOCAITY_URL, DEFAULT_REPLICATE_URL
from socaity.api.text.chat.llama3.meta_llama3_schema import MetaLlama3_Input


srvc_meta_llama_3_8b = APIClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/meta-llama-3-8b",
        "replicate": f"{DEFAULT_REPLICATE_URL}/meta/meta-llama-3-8b",
        "socaity_local": f"http://localhost:8000/v0/meta-llama-3-8b",
    },
    service_name="meta-llama-3-8b",
    model_description=AIModelDescription(
        model_name="meta-llama-3-8b",
        model_domain_tags=[ModelDomainTag.TEXT],
        model_tags=["llm"],
    )
)

srvc_meta_llama_3_8b.add_endpoint(endpoint_route="/chat", query_params=MetaLlama3_Input(), refresh_interval_s=0.5)

srvc_meta_llama_3_70b = APIClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/meta-llama-3-70b",
        "replicate": f"{DEFAULT_REPLICATE_URL}/meta/meta-llama-3-70b",
        "socaity_local": f"http://localhost:8000/v0/meta-llama-3-70b",
    },
    service_name="meta-llama-3-70b",
    model_description=AIModelDescription(
        model_name="meta-llama-3-70b",
        model_domain_tags=[ModelDomainTag.TEXT],
        model_tags=["llm"]
    )
)

srvc_meta_llama_3_70b.add_endpoint(endpoint_route="/chat", query_params=MetaLlama3_Input(), refresh_interval_s=0.5)


import random

import media_toolkit as mt
from typing import Union, Tuple, List

from fastsdk import fastSDK, fastJob
from fastsdk.jobs.threaded.internal_job import InternalJob
from media_toolkit import ImageFile
from socaity.api.image.text2img.flux_schnell.flux_schnell_service_client import srvc_flux_schnell
from socaity.api.image.text2img.text2image import _BaseText2Image
from socaity.api.utils import execute_job_function


@fastSDK(api_client=srvc_flux_schnell)
class FluxSchnell(_BaseText2Image):
    @fastJob
    def _text2img(
            self,
            job: InternalJob,
            text: str,
            aspect_ratio: Union[str, Tuple[int, int]] = "1:1",
            num_outputs: int = 1,
            num_inference_steps: int = 4,
            seed: int = None,
            output_format: str = 'png',
            disable_safety_checker: bool = True,
            go_fast: bool = False
    ) -> Union[mt.ImageFile, List[mt.ImageFile], None]:
        if seed is None:
            seed = random.Random().randint(0, 1000000)

        if isinstance(aspect_ratio, tuple):
            aspect_ratio = f"{aspect_ratio[0]}:{aspect_ratio[1]}"
        if not aspect_ratio or not isinstance(aspect_ratio, str):
            aspect_ratio = "1:1"

        endpoint_request = job.request(
            endpoint_route="text2img",
            prompt=text, aspect_ratio=aspect_ratio, num_outputs=num_outputs, num_inference_steps=num_inference_steps,
            seed=seed, output_format=output_format, disable_safety_checker=disable_safety_checker, go_fast=go_fast
        )
        endpoint_request.wait_until_finished()

        if endpoint_request.error is not None:
            raise Exception(f"Error in text2image with flux_schnell: {endpoint_request.error}")

        res = endpoint_request.get_result()
        if isinstance(res, list):
            if len(res) == 1:
                return ImageFile().from_any(res[0])
            return [ImageFile().from_any(img) for img in res]

        return ImageFile().from_any(res)

    def text2img(
            self,
            text: str,
            aspect_ratio: Union[str, Tuple[int, int]] = "1:1",
            num_outputs: int = 1,
            num_inference_steps: int = 4,
            seed: int = None,
            output_format: str = 'png',
            disable_safety_checker: bool = True,
            go_fast: bool = False
    ) -> InternalJob:
        """
        Converts text to an image.
        :param text: The prompt to be converted to an image.
        """
        return execute_job_function(self._text2img, pam_locals=locals())




from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.api_client import APIClient
from media_toolkit import ImageFile
from socaity.settings import DEFAULT_SOCAITY_URL, DEFAULT_REPLICATE_URL

srvc_blip = APIClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/blip",
        "socaity_local": "http://localhost:8000/v0/blip",
        "replicate": { "version": "2e1dddc8621f72155f24cf2e0adbde548458d3cab9f00c0139eea840d0ac4746"},
    },
    service_name="blip",
    model_description=AIModelDescription(
        model_name="blip",
        model_domain_tags=[ModelDomainTag.IMAGE, ModelDomainTag.TEXT]
    )
)

# Endpoint definitions
from pydantic import BaseModel, Field


class BlipInput(BaseModel):
    task: str = Field(default="image_captioning", description="The task to perform.")
    question: str = Field(default="", description="The question for the visual question answering task")
    caption: str = Field(default="", description="Type caption for the input image for image text matching task.")


# Add endpoint to the service client
srvc_blip.add_endpoint(
    endpoint_route="/capture",
    query_params=BlipInput(),
    file_params={"image": ImageFile},
    refresh_interval_s=0.3
)



from socaity import SpeechCraft
import os

test_file_1 = 'test_files/text2speech/voice_clone_test_voice_1.wav'
test_file_2 = 'test_files/text2speech/voice_clone_test_voice_2.wav'

sample_text = "I love Jacobo. He is the best man in the world and has big eggs."
out_dir = "test_files/output/text2speech"

sc = SpeechCraft()

def test_text2voice():
    t2v_job = sc.text2voice(sample_text)
    audio = t2v_job.get_result()
    audio.save(f"{out_dir}/en_speaker_3_i_love_socaity.wav")


## test voice cloning
def test_voice2embedding():
    v2e_job = sc.voice2embedding(audio_file=test_file_1, voice_name="hermine", save=True)
    embedding = v2e_job.get_result()
    audio_with_cloned_voice = sc.text2voice(sample_text, voice=embedding).get_result()
    audio_with_cloned_voice.save(f"{out_dir}/hermine_i_love_socaity.wav")

def test_voice2voice():
    # test voice2voice
    v2v_job = sc.voice2voice(
        audio_file=test_file_2,
        voice_name="hermine"
    )
    v2v_audio = v2v_job.get_result()
    v2v_audio.save(f"{out_dir}/benni.wav")



if __name__ == "__main__":
    test_text2voice()
    test_voice2embedding()
    test_voice2voice()



from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.api_client import APIClient
from socaity.settings import DEFAULT_SOCAITY_URL


srvc_hunyuan_video = APIClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/hunyuan-video",
        "socaity_local": "http://localhost:8000/v0/hunyuan-video",
        "replicate": {
            "version": "6c9132aee14409cd6568d030453f1ba50f5f3412b844fe67f78a9eb62d55664f"
        }
    },
    service_name="hunyuan-video",
    model_description=AIModelDescription(
        model_name="hunyuan-video",
        model_domain_tags=[ModelDomainTag.VIDEO, ModelDomainTag.TEXT]
    )
)

# Endpoint definitions

from pydantic import BaseModel, Field


class HunyuanVideoText2ImgPostParams(BaseModel):
    prompt: str = Field(default="")
    width: int = Field(default=864)
    height: int = Field(default=480)
    video_length: int = Field(default=129)  # in frames
    infer_steps: int = Field(default=50)
    seed: int = Field(default=None)
    embedded_guidance_scale: int = Field(default=False)


# ToDo: support pydantic schemas for default values..
srvc_hunyuan_video.add_endpoint(
    endpoint_route="/text2video",
    query_params=HunyuanVideoText2ImgPostParams(),
    refresh_interval_s=5
)


import os

from socaity import FluxSchnell


#fluxs = FluxSchnell(service="replicate", api_key=os.getenv("REPLICATE_API_KEY", None))
#fluxs = FluxSchnell(service="socaity_local", api_key=os.getenv("SOCAITY_API_KEY", None))
fluxs = FluxSchnell()

#from socaity import text2img
#my_generated_image = text2img("An elephant swimming in a lake")


def test_text2img():
    prompt = (
        """
A massive beautiful sci-fi robot giant sequoia tree. The full tree including deep roots is showcased. The canopy layer branches into three big branches. The stem is kept shorter. 
The roots look like cables. The stem like a server farm. Branches are fine and robotic structures.

The purpose of the image is to show different technology layers of a companys tech stack.
The root layer will represent the foundational technology blending in typical cloud, server, docker and kubernetes icons.
The stem represents the API and SDK layer.
The canopy layer represents key operating pillars of the company including youtube icon, wordpress, agentic heads, and a sci-fi fashion store.
The upper emergent layer, leaves spaces for additional concrete products.
Theres a swift seperation line between the layers like in an rainforest ecosystem illustration.
The artwork is minimalistic yet striking, showcasing a vibrant deep-purple and neon-green lime palette, rendered in an anime-style illustration with 4k detail. 
Influenced by the artistic styles of Simon Kenny, Giorgetto Giugiaro, Brian Stelfreeze, and Laura Iverson
        """
    )
    fj = fluxs.text2img(
        text=prompt, aspect_ratio="9:16", num_outputs=4, num_inference_steps=4, output_format="png",
        disable_safety_checker=True, go_fast=False
    )
    imgs = fj.get_result()
    if not isinstance(imgs, list):
        imgs = [imgs]

    for i, img in enumerate(imgs):
        img.save(f"test_files/output/text2img/test_fluxs_text2img_{i}.png")

if __name__ == "__main__":
    test_text2img()


# settings first to set environment variables
from socaity.settings import API_KEYS
from socaity.api import *

from fastsdk import gather_generator, gather_results
from fastsdk.registry import Registry

from media_toolkit import MediaFile, ImageFile, VideoFile, AudioFile




from typing import Union

from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk.fast_sdk import fastSDK, fastJob
from fastsdk.utils import get_function_parameters_as_dict
from media_toolkit import ImageFile
from .sam2_service_client import srvc_sam2
from socaity.api.image.img2img.image_segmentation.i_image_segmentation import _BaseSegmentation


@fastSDK(api_client=srvc_sam2)
class Sam2(_BaseSegmentation):
    """
    Base implementation for SAM 2, the Segment Anything v2 model from Meta.
    """
    @fastJob
    def _segment(self, job,
                 image: Union[str, bytes, ImageFile],
                 points_per_side: int = 32,
                 pred_iou_thresh: float = 0.88,
                 stability_score_thresh: float = 0.95,
                 use_m2m: bool = True,
                 **kwargs) -> dict:
        """
        Perform image segmentation using SAM 2.
        :param image: URL or path to the input image.
        :param points_per_side: Number of points per side for segmentation.
        :param pred_iou_thresh: Prediction IoU threshold.
        :param stability_score_thresh: Stability score threshold.
        :param use_m2m: Whether to enable model-to-model interaction.
        :return: Segmentation results as a dictionary.
        """
        response = job.request(
            endpoint_route="/segment",
            image=image,
            points_per_side=points_per_side,
            pred_iou_thresh=pred_iou_thresh,
            stability_score_thresh=stability_score_thresh,
            use_m2m=use_m2m,
            **kwargs,
        )
        res = response.get_result()
        if response.error:
            raise Exception(f"Error in segmentation: {response.error}")

        result = {}
        combined_mask = res.get("combined_mask", None)
        individual_masks = res.get("individual_masks", None)
        if combined_mask:
            result["combined_mask"] = ImageFile().from_any(combined_mask)
        if individual_masks:
            result["individual_masks"] = [ImageFile().from_any(mask) for mask in individual_masks]

        return result

    def segment(self,
        image: Union[str, bytes, ImageFile],
        points_per_side: int = 32,
        pred_iou_thresh: float = 0.88,
        stability_score_thresh: float = 0.95,
        use_m2m: bool = True,
        *args, **kwargs) -> InternalJob:
        """
        Perform image segmentation using SAM 2.
        :param image: URL or path to the input image.
        """
        pams = get_function_parameters_as_dict(
            self._segment,
            exclude_param_names="job",
            func_kwargs=locals()
        )

        return self._segment(**pams)




from typing import Union, List

import media_toolkit as mt

class _BaseText2Video:
    def text2video(self, text, *args, **kwargs) -> Union[mt.VideoFile, List[mt.VideoFile], None]:
        """
        Converts text to an image
        :param text: The text to convert to an image
        :return: The image
        """
        raise NotImplementedError("Please implement this method")


# Factory method for generalized model_hosting_info calling
def text2video(text, model="hunyuan_video", service="socaity", *args, **kwargs) -> Union[mt.VideoFile, List[mt.VideoFile], None]:
    if model == "hunyuan_video":
        from .hunyuan_video.hunyuan_video import HunyuanVideo
        s = HunyuanVideo(service=service)
        return s.text2video(text, *args, **kwargs)


    return None



from enum import Enum

from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.api_client import APIClient
from media_toolkit import AudioFile
from socaity.settings import DEFAULT_SOCAITY_URL, DEFAULT_REPLICATE_URL

srvc_whisper = APIClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/insanely-fast-whisper",
        "socaity_local": "http://localhost:8000/v0/insanely-fast-whisper",
        "replicate": { "version": "3ab86df6c8f54c11309d4d1f930ac292bad43ace52d10c80d87eb258b3c9f79c"},
    },
    service_name="insanely-fast-whisper",
    model_description=AIModelDescription(
        model_name="insanely-fast-whisper",
        model_domain_tags=[ModelDomainTag.IMAGE, ModelDomainTag.TEXT]
    ),
    upload_to_cloud_threshold_mb=0
)

class WhisperTasks(Enum):
    TRANSCRIBE = "transcribe"
    TRANSLATE = "translate"

class WhisperTimeStamp(Enum):
    CHUNK = "chunk"
    WORD = "word"


# Add endpoint to the service client
srvc_whisper.add_endpoint(
    endpoint_route="/transcribe",
    query_params= {
        "task": str,
        "language": str,
        "batch_size": int,
        "timestamp": str
    },
    file_params={"audio": AudioFile},
    refresh_interval_s=0.5
)



import os

from socaity.api.text import InsanelyFastWhisper

test_audio = "test_files/audio/potter_to_hermine.wav"
genai = InsanelyFastWhisper()
# fluxs = FluxSchnell(service="socaity_local", api_key=os.getenv("SOCAITY_API_KEY", None))

def test_transcribe():
    fj = genai.transcribe(audio=test_audio)
    generated_text = fj.get_result()
    print(generated_text)


if __name__ == "__main__":
    test_transcribe()



import os
from fastsdk.settings import API_KEYS

# The basis URL of all SOCAITY services.
# If not specified differently when a class is instantiated requests are sent to this URL.
DEFAULT_SOCAITY_URL = os.environ.get("SOCAITY_API_URL", "https://api.socaity.ai/v0")
DEFAULT_REPLICATE_URL = os.environ.get("REPLICATE_API_URL", "https://api.replicate.com/v1/models")


# For services hosted on runpod, an API key is required.
# If a service client calls an endpoint with one of those, the API key is added to the request in the header.
API_KEYS["socaity"] = os.getenv("SOCAITY_API_KEY", None)
API_KEYS["socaity_local"] = os.getenv("SOCAITY_API_KEY", None)  # debugging
API_KEYS["runpod"] = os.environ.get("RUNPOD_API_KEY", None)
API_KEYS["replicate"] = os.environ.get("REPLICATE_API_KEY", None)




from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.api_client import APIClient
from socaity.settings import DEFAULT_SOCAITY_URL, DEFAULT_REPLICATE_URL
from socaity.api.text.chat.deepseek.deepseek_r1_schema import DeepSeekR1_Input


srvc_deep_seek_r1 = APIClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/deepseek-r1",
        "replicate": f"{DEFAULT_REPLICATE_URL}/deepseek-ai/deepseek-r1",
    },
    service_name="deepseek-r1",
    model_description=AIModelDescription(
        model_name="deepseek-r1",
        model_domain_tags=[ModelDomainTag.TEXT],
        model_tags=["llm"],
    )
)

srvc_deep_seek_r1.add_endpoint(endpoint_route="/chat", query_params=DeepSeekR1_Input(), refresh_interval_s=0.5)



import random

import media_toolkit as mt
from typing import Union, List

from fastsdk import fastSDK, fastJob
from fastsdk.jobs.threaded.internal_job import InternalJob
from media_toolkit import VideoFile
from socaity.api.video.text2video.hunyuan_video.hunyuan_video_service_client import srvc_hunyuan_video
from socaity.api.video.text2video.text2video import _BaseText2Video


@fastSDK(api_client=srvc_hunyuan_video)
class HunyuanVideo(_BaseText2Video):
    @fastJob
    def _text2video(
            self,
            job: InternalJob,
            prompt: str,
            width: int = 854,
            height: int = 480,
            video_length: int = 129,  # in frames
            infer_steps: int = 50,
            seed: int = None,
            embedded_guidance_scale: int = 6
    ) -> Union[mt.VideoFile, List[mt.VideoFile], None]:
        if not seed or not isinstance(seed, int):
            seed = random.Random().randint(0, 10000)

        if (video_length-1) % 4 != 0:
            raise ValueError(f"video_length-1 must be multiple of 4 got {video_length}")

        endpoint_request = job.request(
            endpoint_route="text2video",
            prompt=prompt,
            width=width,
            height=height,
            video_length=video_length,
            infer_steps=infer_steps,
            seed=seed,
            embedded_guidance_scale=embedded_guidance_scale
        )

        endpoint_request.wait_until_finished()
        if endpoint_request.error is not None:
            raise Exception(f"Error in text2video with huanyan_video: {endpoint_request.error}")

        res = endpoint_request.get_result()
        if isinstance(res, list) and not isinstance(res, str):
            if len(res) == 1:
                return VideoFile().from_any(res[0])
            return [VideoFile().from_any(vid) for vid in res]

        return VideoFile().from_any(res)

    def text2video(
            self,
            text: str,
            width: int = 854,
            height: int = 480,
            video_length: int = 129,  # in frames
            infer_steps: int = 50,
            seed: int = None,
            embedded_guidance_scale: int = 6,
            *args,
            **kwargs
    ) -> InternalJob:
        """
        Converts text to an image.
        :param text: The prompt to be converted to an image.
        """
        return self._text2video(
            prompt=text,
            width=width,
            height=height,
            video_length=video_length,
            infer_steps=infer_steps,
            seed=seed,
            embedded_guidance_scale=embedded_guidance_scale
        )


from abc import abstractmethod
from typing import Union, List

import media_toolkit as mt
from fastsdk.jobs.threaded.internal_job import InternalJob
from media_toolkit import AudioFile
from socaity.api.utils import get_model_instance


class _BaseText2Voice:

    @abstractmethod
    def text2voice(self, text: str, *args, **kwargs) -> Union[mt.AudioFile, List[mt.AudioFile], None]:
        """
        Converts text to an image
        :param text: The text to convert to an image
        :return: The image
        """
        raise NotImplementedError("Please implement this method")


def text2voice(
        text: str, model="speechcraft", service="socaity", wait_for_result: bool = False, *args, **kwargs
) -> Union[InternalJob, AudioFile, List[AudioFile], str, None]:
    """
    Converts a given text to an spoken audio file.
    :param text: The text to convert to audio.
    :param model: The model to use for the conversion.
    :param service: The service to use for the conversion.
    :param wait_for_result: Whether to wait for the result. If False (default), returns the job.
    """
    from .speechcraft import SpeechCraft
    names_cl = {"speechcraft": SpeechCraft}
    mdl = get_model_instance(names_cl, model_name=model, service=service, default_instance=SpeechCraft)
    job = mdl.text2voice(text=text, *args, **kwargs)
    if wait_for_result:
        return job.get_result()
    return job



from socaity import TencentPhotoMaker, ImageFile
import os

genai = TencentPhotoMaker(service="replicate", api_key=os.getenv("REPLICATE_API_KEY", None))

def test_TencentPhotoMaker():
    image = "test_files/face2face/test_face_1.jpg"
    fj = genai.create_socaity_avatar(input_image=image, num_outputs=3)
    avatars = fj.get_result()
    if not isinstance(avatars, list):
        avatars = [avatars]

    for i, avatar in enumerate(avatars):
        avatar = ImageFile().from_any(avatar)
        avatar.save(f"test_files/output/tencent_photomaker/avatar_{i}.jpg")


if __name__ == "__main__":
    test_TencentPhotoMaker()